package domains

import (
	"bytes"
	"fmt"

	"tuxpa.in/t/wm/src/copies"
	"tuxpa.in/t/wm/src/sock"
)

type Query struct {
	UseNames bool

	inject

	cmd
}

func (n *Query) Run(msg *sock.Msg) ([]byte, error) {
	if !msg.HasNext() {
		return nil, &copies.ErrMissingArguments{}
	}
	for {
		ok, err := n.parse(msg)
		if err != nil {
			return nil, err
		}
		if !ok {
			break
		}
	}
	switch n.Command {
	case "desktops":
		return n.desktops(msg)
	default:
		return nil, &copies.ErrTODO{}
	}
}

func (n *Query) desktops(msg *sock.Msg) ([]byte, error) {
	o := new(bytes.Buffer)
	o.WriteString("hi there")
	return o.Bytes(), nil
}

func (n *Query) parse(msg *sock.Msg) (bool, error) {
	if !msg.HasNext() {
		return false, &copies.ErrMissingArguments{}
	}
	arg := msg.Next()
	switch arg {
	case "--desktop", "-d":
	case "--desktops", "-D":
		return n.readCommand(msg, "desktops")
	case "--monitor", "-m":
	case "--monitors", "-M":
		return n.readCommand(msg, "monitors")
	case "--names":
		n.UseNames = true
	case "--node", "-n":
	case "--nodes", "-N":
		return n.readCommand(msg, "nodes")
	case "--tree", "-T":
		return n.readCommand(msg, "tree")
	default:
		return false, fmt.Errorf(`unknown option: '%s'`, arg)
	}
	return msg.HasNext(), nil
}
